@@ -6,12 +6,7 @@
 #' @param xlab Label for X
 #' @param ylab Label for Y
 #' @param mlab Label for M
-#' @param border.width Size of node borders (defaults to 2).
-#' @param edge.label.cex Text size.
-#' @param edge.color Color of the path arrows. (Set NULL to color negative paths
-#' red, and positive paths green.)
-#' @param fade Should edges fade to white? (Defaults to FALSE.)
-#' @param level "Confidence" level for credible intervals. (Defaults to .99.)
+#' @param level "Confidence" level for credible intervals. (Defaults to .95.)
 #' @param text Should additional parameter values be displayed?
 #' (Defaults to FALSE.)
 #' @param template Should an empty template diagram be plotted?
@@ -28,17 +23,16 @@
 #' be used to draw a template diagram of the mediation model by setting
 #' \code{template = TRUE}.
 #'
+#' To modify various settings of the underlying qgraph object, see
+#' \code{\link[qgraph]{qgraph}}.
+#'
 #' @examples
 #' # Draw a template path diagram of the mediation model
 #' mlm_path_plot(template = TRUE)
 #'
 #' @export
 mlm_path_plot <- function(mod = NULL, xlab = "X", ylab = "Y", mlab = "M",
-                          border.width = 2,
-                          edge.label.cex = 1,
-                          edge.color = "black",
-                          fade = FALSE,
-                          level = .99,
+                          level = .95,
                           text = FALSE,
                           template = FALSE,
                           id = NULL,
@@ -49,6 +43,7 @@ mlm_path_plot <- function(mod = NULL, xlab = "X", ylab = "Y", mlab = "M",
         stop("qgraph package needed for this function. Please install it.",
              call. = FALSE)
     }
+
     if (template) {
         # If user wants a template diagram
         edgelabels <- c(" \n  a  \n ", " \n  b  \n ", " \n  c'  \n ")
@@ -76,13 +71,13 @@ mlm_path_plot <- function(mod = NULL, xlab = "X", ylab = "Y", mlab = "M",
                                 level = level)
         }
 
-        sfit <- subset(sfit, select = c("Parameter", "Mean", "ci_lwr", "ci_upr"))
-        a <- sfit[sfit$Parameter == "a", c("Mean", "ci_lwr", "ci_upr")]
-        b <- sfit[sfit$Parameter == "b", c("Mean", "ci_lwr", "ci_upr")]
-        cp <- sfit[sfit$Parameter == "cp", c("Mean", "ci_lwr", "ci_upr")]
-        ab <- sfit[sfit$Parameter == "ab", c("Mean", "ci_lwr", "ci_upr")]
-        c <- sfit[sfit$Parameter == "c", c("Mean", "ci_lwr", "ci_upr")]
-        pme <- sfit[sfit$Parameter == "pme", c("Mean", "ci_lwr", "ci_upr")]
+        sfit <- subset(sfit, select = c(1,2,5,6))
+        a <- sfit[sfit$Parameter == "a", c(2:4)]
+        b <- sfit[sfit$Parameter == "b", c(2:4)]
+        cp <- sfit[sfit$Parameter == "cp", c(2:4)]
+        ab <- sfit[sfit$Parameter == "ab", c(2:4)]
+        c <- sfit[sfit$Parameter == "c", c(2:4)]
+        pme <- sfit[sfit$Parameter == "pme", c(2:4)]
 
         edgelabels <- c(
             paste0("\n", a[1], " \n   [", a[2], ", ", a[3], "]   \n"),
@@ -93,22 +88,29 @@ mlm_path_plot <- function(mod = NULL, xlab = "X", ylab = "Y", mlab = "M",
                                  0, 1, 0,
                                  a[1], cp[1] ,1)), byrow=T, nrow = 3)
     }
+
+    # Set bmlm default args to qgraph
+    qargs <- list(...)
+    qargs$input <- x
+    qargs$labels <- c(mlab, ylab, xlab)
+    if (is.null(qargs$border.width)) qargs$border.width <- 2
+    if (is.null(qargs$edge.label.cex)) qargs$edge.label.cex <- 1.2
+    if (is.null(qargs$edge.color)) qargs$edge.color <- "black"
+    if (is.null(qargs$vsize)) qargs$vsize <- 16
+    if (is.null(qargs$vsize2)) qargs$vsize2 <- 12
+    if (is.null(qargs$asize)) qargs$asize <- 4
+    if (is.null(qargs$esize)) qargs$esize <- 4
+    if (is.null(qargs$label.norm)) qargs$label.norm <- "OOOOOO"
+    if (is.null(qargs$edge.labels)) qargs$edge.labels <- edgelabels
+    if (is.null(qargs$mar)) qargs$mar <- c(4, 4, 4, 4)
+    if (is.null(qargs$fade)) qargs$fade <- FALSE
+    if (is.null(qargs$layout)) qargs$layout <- "circle"
+    if (is.null(qargs$shape)) qargs$shape <- "rectangle"
+    if (is.null(qargs$weighted)) qargs$weighted <- FALSE
+
     # Create plot
-    qgraph::qgraph(x, layout = "circle",
-                   shape = "rectangle",
-                   vsize = 16,
-                   vsize2 = 12,
-                   labels = c(mlab, ylab, xlab),
-                   label.norm = "OOOOOO",
-                   border.width = border.width,
-                   edge.labels = edgelabels,
-                   edge.label.cex = edge.label.cex,
-                   fade = fade,
-                   asize = 10,
-                   esize = 10,
-                   mar = c(4, 4, 4, 4),
-                   edge.color = edge.color,
-                   ...)
+    do.call(qgraph::qgraph, qargs)
+
     # If ID is specified, note this on plot
     if (!is.null(id)) {
         graphics::text(1.25, 1.25, paste0("ID: ", id), font = 2)
@@ -132,7 +134,7 @@ mlm_path_plot <- function(mod = NULL, xlab = "X", ylab = "Y", mlab = "M",
 #'
 #' @param mod A Stanfit model estimated with \code{mlm()}.
 #' @param type Type of the plot, \code{hist} or \code{coefplot}.
-#' @param level X level for Credible Intervals. (Defaults to .99.)
+#' @param level X level for Credible Intervals. (Defaults to .95.)
 #' @param color Color (and fill) for plots.
 #' @param p_shape Shape of points for coefplot.
 #' @param p_size Size of points for coefplot.
@@ -143,7 +145,7 @@ mlm_path_plot <- function(mod = NULL, xlab = "X", ylab = "Y", mlab = "M",
 #'
 #' @author Matti Vuorre \email{mv2521@columbia.edu}
 #'
-#' @details The point estimate for the coefficient plot is the posterior median.
+#' @details The point estimate for the coefficient plot is the posterior mean.
 #'
 #' @import ggplot2
 #'
@@ -153,7 +155,7 @@ mlm_pars_plot <- function(mod = NULL,
                           color = "black",
                           p_shape = 15,
                           p_size = 1.2,
-                          level = 0.99,
+                          level = 0.95,
                           nrow = 3,
                           pars = c("a", "b", "cp", "corrab", "ab", "c", "pme")){
 
@@ -168,6 +170,12 @@ mlm_pars_plot <- function(mod = NULL,
         stop("Model is not a stanfit object.", call. = FALSE)
     }
 
+    Theme <- theme_bw() +
+        theme(axis.title = element_blank(),
+              axis.ticks = element_line(size = .3),
+              panel.background = element_rect(color="gray50"),
+              panel.grid = element_blank())
+
     d <- as.data.frame(mod, pars = pars)
     d <- reshape2::melt(d)
 
@@ -187,25 +195,67 @@ mlm_pars_plot <- function(mod = NULL,
                   panel.grid = element_blank(),
                   strip.background = element_rect(fill = NA, colour = NA),
                   strip.text.x = element_text(face = "bold"))
-    } else {
+    } else if (type == "coef") {
+        td <- d
         d <- dplyr::group_by_(d, "variable")
         d <- dplyr::summarize_(
             d,
-            m = ~median(value),  # Formulas allow non-standard evaluation
+            m = ~mean(value),
             lwr = ~stats::quantile(value, probs = .5 - level/2),
             upr = ~stats::quantile(value, probs = .5 + level/2)
             )
-        p1 <- ggplot2::ggplot(d, aes_string(x = "variable", y = "m")) +
-            geom_hline(yintercept = 0, lty = 2, size = .3) +
-            geom_point(aes_string(y="m"), shape = p_shape, size = p_size) +
-            geom_linerange(aes_string(y="m", ymin = "lwr", ymax = "upr"),
-                           size = p_size / 4.5) +
-            coord_flip() +
-            theme_bw() +
-            theme(axis.title = element_blank(),
-                  axis.ticks = element_line(size = .3),
-                  panel.background = element_rect(color="gray50"),
-                  panel.grid = element_blank())
+        d <- as.data.frame(d)
+        # If parameter is varying, do stuff
+        d$var <- grepl("u_", d[, "variable"])
+        par_var <- any(d$var)
+        if (par_var) {
+            # Reorder estimates on value
+            d[, "variable"] <- stats::reorder(d[, "variable"], d[, "m"], mean)
+            # Highlight average effect if present
+            d$hl <- ifelse(d$var, "a", "b")
+            p1 <- ggplot2::ggplot(d, aes_string(x = "variable",
+                                                y = "m",
+                                                color = "hl")) +
+                scale_color_manual(values = c(color, "tomato2"), guide = "none") +
+                geom_hline(yintercept = 0, lty = 2, size = .3) +
+                geom_point(aes_string(y="m"), shape = p_shape, size = p_size) +
+                geom_linerange(aes_string(y="m", ymin = "lwr", ymax = "upr"),
+                               size = p_size / 4.5) +
+                Theme + theme(axis.text.x = element_blank(),
+                              axis.ticks.x = element_blank())
+        } else {
+            p1 <- ggplot2::ggplot(d, aes_string(x = "variable", y = "m")) +
+                geom_hline(yintercept = 0, lty = 2, size = .3) +
+                geom_point(aes_string(y="m"), shape = p_shape, size = p_size) +
+                geom_linerange(aes_string(y="m", ymin = "lwr", ymax = "upr"),
+                               size = p_size / 4.5) +
+                Theme
+        }
+    } else {  # Violin plot
+        # If parameter is varying, do stuff
+        d$var <- grepl("u_", d[, "variable"])
+        par_var <- any(d$var)
+        if (par_var) {
+            # Reorder estimates on value
+            d[, "variable"] <- stats::reorder(d[, "variable"], d[, "value"], mean)
+            # Highlight average effect if present
+            d$hl <- ifelse(d$var, "a", "b")
+            p1 <- ggplot2::ggplot(d, aes_string(x = "variable",
+                                                y = "value",
+                                                fill = "hl")) +
+                scale_fill_manual(values = c(color, "tomato2"), guide = "none") +
+                geom_hline(yintercept = 0, lty = 2, size = .3) +
+                geom_violin(data = d, aes_string(y="value"),
+                            col = NA) +
+                Theme + theme(axis.text.x = element_blank(),
+                              axis.ticks.x = element_blank())
+        } else {
+            p1 <- ggplot2::ggplot(d, aes_string(x = "variable", y = "m")) +
+                geom_hline(yintercept = 0, lty = 2, size = .3) +
+                geom_violin(data = d, aes_string(y="value"),
+                            col = NA, fill = color) +
+                Theme
+        }
     }
     return(p1)
 }
